@@ -30,13 +30,12 @@ from six.moves import map, zip
 
 from keystone.common import driver_hints
 from keystone import exception
-from keystone.i18n import _, _LW
+from keystone.i18n import _
 
 
 LOG = log.getLogger(__name__)
 
 LDAP_VALUES = {'TRUE': True, 'FALSE': False}
-CONTROL_TREEDELETE = '1.2.840.113556.1.4.805'
 LDAP_SCOPES = {'one': ldap.SCOPE_ONELEVEL,
                'sub': ldap.SCOPE_SUBTREE}
 LDAP_DEREF = {'always': ldap.DEREF_ALWAYS,
@@ -55,6 +54,10 @@ DN_ONLY = ['1.1']
 
 _utf8_encoder = codecs.getencoder('utf-8')
 
+# FIXME(knikolla): This enables writing to the LDAP backend
+# Only enabled during tests and unsupported
+WRITABLE = False
+
 
 def utf8_encode(value):
     """Encode a basestring to UTF-8.
@@ -428,8 +431,8 @@ class LDAPHandler(object):
     def connect(self, url, page_size=0, alias_dereferencing=None,
                 use_tls=False, tls_cacertfile=None, tls_cacertdir=None,
                 tls_req_cert=ldap.OPT_X_TLS_DEMAND, chase_referrals=None,
-                debug_level=None, use_pool=None, pool_size=None,
-                pool_retry_max=None, pool_retry_delay=None,
+                debug_level=None, conn_timeout=None, use_pool=None,
+                pool_size=None, pool_retry_max=None, pool_retry_delay=None,
                 pool_conn_timeout=None, pool_conn_lifetime=None):
         raise exception.NotImplemented()  # pragma: no cover
 
@@ -475,14 +478,6 @@ class LDAPHandler(object):
     def modify_s(self, dn, modlist):
         raise exception.NotImplemented()  # pragma: no cover
 
-    @abc.abstractmethod
-    def delete_s(self, dn):
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
-    def delete_ext_s(self, dn, serverctrls=None, clientctrls=None):
-        raise exception.NotImplemented()  # pragma: no cover
-
 
 class PythonLDAPHandler(LDAPHandler):
     """LDAPHandler implementation which calls the python-ldap API.
@@ -496,8 +491,8 @@ class PythonLDAPHandler(LDAPHandler):
     def connect(self, url, page_size=0, alias_dereferencing=None,
                 use_tls=False, tls_cacertfile=None, tls_cacertdir=None,
                 tls_req_cert=ldap.OPT_X_TLS_DEMAND, chase_referrals=None,
-                debug_level=None, use_pool=None, pool_size=None,
-                pool_retry_max=None, pool_retry_delay=None,
+                debug_level=None, conn_timeout=None, use_pool=None,
+                pool_size=None, pool_retry_max=None, pool_retry_delay=None,
                 pool_conn_timeout=None, pool_conn_lifetime=None):
 
         _common_ldap_initialization(url=url,
@@ -505,7 +500,8 @@ class PythonLDAPHandler(LDAPHandler):
                                     tls_cacertfile=tls_cacertfile,
                                     tls_cacertdir=tls_cacertdir,
                                     tls_req_cert=tls_req_cert,
-                                    debug_level=debug_level)
+                                    debug_level=debug_level,
+                                    timeout=conn_timeout)
 
         self.conn = ldap.initialize(url)
         self.conn.protocol_version = ldap.VERSION3
@@ -560,18 +556,12 @@ class PythonLDAPHandler(LDAPHandler):
     def modify_s(self, dn, modlist):
         return self.conn.modify_s(dn, modlist)
 
-    def delete_s(self, dn):
-        return self.conn.delete_s(dn)
-
-    def delete_ext_s(self, dn, serverctrls=None, clientctrls=None):
-        return self.conn.delete_ext_s(dn, serverctrls, clientctrls)
-
 
 def _common_ldap_initialization(url, use_tls=False, tls_cacertfile=None,
                                 tls_cacertdir=None, tls_req_cert=None,
-                                debug_level=None):
+                                debug_level=None, timeout=None):
     """LDAP initialization for PythonLDAPHandler and PooledLDAPHandler."""
-    LOG.debug("LDAP init: url=%s", url)
+    LOG.debug('LDAP init: url=%s', url)
     LOG.debug('LDAP init: use_tls=%s tls_cacertfile=%s tls_cacertdir=%s '
               'tls_req_cert=%s tls_avail=%s',
               use_tls, tls_cacertfile, tls_cacertdir,
@@ -582,6 +572,10 @@ def _common_ldap_initialization(url, use_tls=False, tls_cacertfile=None,
 
     using_ldaps = url.lower().startswith("ldaps")
 
+    if timeout is not None and timeout > 0:
+        # set network connection timeout
+        ldap.set_option(ldap.OPT_NETWORK_TIMEOUT, timeout)
+
     if use_tls and using_ldaps:
         raise AssertionError(_('Invalid TLS / LDAPS combination'))
 
@@ -617,7 +611,7 @@ def _common_ldap_initialization(url, use_tls=False, tls_cacertfile=None,
         if tls_req_cert in list(LDAP_TLS_CERTS.values()):
             ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, tls_req_cert)
         else:
-            LOG.debug("LDAP TLS: invalid TLS_REQUIRE_CERT Option=%s",
+            LOG.debug('LDAP TLS: invalid TLS_REQUIRE_CERT Option=%s',
                       tls_req_cert)
 
 
@@ -683,8 +677,8 @@ class PooledLDAPHandler(LDAPHandler):
     def connect(self, url, page_size=0, alias_dereferencing=None,
                 use_tls=False, tls_cacertfile=None, tls_cacertdir=None,
                 tls_req_cert=ldap.OPT_X_TLS_DEMAND, chase_referrals=None,
-                debug_level=None, use_pool=None, pool_size=None,
-                pool_retry_max=None, pool_retry_delay=None,
+                debug_level=None, conn_timeout=None, use_pool=None,
+                pool_size=None, pool_retry_max=None, pool_retry_delay=None,
                 pool_conn_timeout=None, pool_conn_lifetime=None):
 
         _common_ldap_initialization(url=url,
@@ -692,7 +686,8 @@ class PooledLDAPHandler(LDAPHandler):
                                     tls_cacertfile=tls_cacertfile,
                                     tls_cacertdir=tls_cacertdir,
                                     tls_req_cert=tls_req_cert,
-                                    debug_level=debug_level)
+                                    debug_level=debug_level,
+                                    timeout=pool_conn_timeout)
 
         self.page_size = page_size
 
@@ -822,14 +817,6 @@ class PooledLDAPHandler(LDAPHandler):
     def modify_s(self, conn, dn, modlist):
         return conn.modify_s(dn, modlist)
 
-    @use_conn_pool
-    def delete_s(self, conn, dn):
-        return conn.delete_s(dn)
-
-    @use_conn_pool
-    def delete_ext_s(self, conn, dn, serverctrls=None, clientctrls=None):
-        return conn.delete_ext_s(dn, serverctrls, clientctrls)
-
 
 class KeystoneLDAPHandler(LDAPHandler):
     """Convert data types and perform logging.
@@ -873,14 +860,15 @@ class KeystoneLDAPHandler(LDAPHandler):
     def connect(self, url, page_size=0, alias_dereferencing=None,
                 use_tls=False, tls_cacertfile=None, tls_cacertdir=None,
                 tls_req_cert=ldap.OPT_X_TLS_DEMAND, chase_referrals=None,
-                debug_level=None, use_pool=None, pool_size=None,
-                pool_retry_max=None, pool_retry_delay=None,
+                debug_level=None, conn_timeout=None, use_pool=None,
+                pool_size=None, pool_retry_max=None, pool_retry_delay=None,
                 pool_conn_timeout=None, pool_conn_lifetime=None):
         self.page_size = page_size
         return self.conn.connect(url, page_size, alias_dereferencing,
                                  use_tls, tls_cacertfile, tls_cacertdir,
                                  tls_req_cert, chase_referrals,
                                  debug_level=debug_level,
+                                 conn_timeout=conn_timeout,
                                  use_pool=use_pool,
                                  pool_size=pool_size,
                                  pool_retry_max=pool_retry_max,
@@ -896,7 +884,7 @@ class KeystoneLDAPHandler(LDAPHandler):
 
     def simple_bind_s(self, who='', cred='',
                       serverctrls=None, clientctrls=None):
-        LOG.debug("LDAP bind: who=%s", who)
+        LOG.debug('LDAP bind: who=%s', who)
         who_utf8 = utf8_encode(who)
         cred_utf8 = utf8_encode(cred)
         return self.conn.simple_bind_s(who_utf8, cred_utf8,
@@ -904,7 +892,7 @@ class KeystoneLDAPHandler(LDAPHandler):
                                        clientctrls=clientctrls)
 
     def unbind_s(self):
-        LOG.debug("LDAP unbind")
+        LOG.debug('LDAP unbind')
         return self.conn.unbind_s()
 
     def add_s(self, dn, modlist):
@@ -942,9 +930,12 @@ class KeystoneLDAPHandler(LDAPHandler):
                 attrlist_utf8 = None
             else:
                 attrlist_utf8 = list(map(utf8_encode, attrlist))
-            ldap_result = self.conn.search_s(base_utf8, scope,
-                                             filterstr_utf8,
-                                             attrlist_utf8, attrsonly)
+            try:
+                ldap_result = self.conn.search_s(base_utf8, scope,
+                                                 filterstr_utf8,
+                                                 attrlist_utf8, attrsonly)
+            except ldap.SIZELIMIT_EXCEEDED:
+                raise exception.LDAPSizeLimitExceeded()
 
         py_result = convert_ldap_result(ldap_result)
 
@@ -1026,9 +1017,9 @@ class KeystoneLDAPHandler(LDAPHandler):
                     # Exit condition no more data on server
                     break
             else:
-                LOG.warning(_LW('LDAP Server does not support paging. '
-                                'Disable paging in keystone.conf to '
-                                'avoid this message.'))
+                LOG.warning('LDAP Server does not support paging. '
+                            'Disable paging in keystone.conf to '
+                            'avoid this message.')
                 self._disable_paging()
                 break
         return res
@@ -1067,17 +1058,6 @@ class KeystoneLDAPHandler(LDAPHandler):
             for op, kind, values in ldap_modlist]
         return self.conn.modify_s(dn_utf8, ldap_modlist_utf8)
 
-    def delete_s(self, dn):
-        LOG.debug("LDAP delete: dn=%s", dn)
-        dn_utf8 = utf8_encode(dn)
-        return self.conn.delete_s(dn_utf8)
-
-    def delete_ext_s(self, dn, serverctrls=None, clientctrls=None):
-        LOG.debug('LDAP delete_ext: dn=%s serverctrls=%s clientctrls=%s',
-                  dn, serverctrls, clientctrls)
-        dn_utf8 = utf8_encode(dn)
-        return self.conn.delete_ext_s(dn_utf8, serverctrls, clientctrls)
-
     def __exit__(self, exc_type, exc_val, exc_tb):
         """Exit runtime context, unbind LDAP."""
         self.unbind_s()
@@ -1123,7 +1103,6 @@ class BaseLdap(object):
     DEFAULT_OBJECTCLASS = None
     DEFAULT_FILTER = None
     DEFAULT_EXTRA_ATTR_MAPPING = []
-    DUMB_MEMBER_DN = 'cn=dumb,dc=nonexistent'
     NotFound = None
     notfound_arg = None
     options_name = None
@@ -1147,6 +1126,7 @@ class BaseLdap(object):
         self.attribute_mapping = {}
         self.chase_referrals = conf.ldap.chase_referrals
         self.debug_level = conf.ldap.debug_level
+        self.conn_timeout = conf.ldap.connection_timeout
 
         # LDAP Pool specific attribute
         self.use_pool = conf.ldap.use_pool
@@ -1187,15 +1167,6 @@ class BaseLdap(object):
             self.ldap_filter = getattr(conf.ldap,
                                        ldap_filter) or self.DEFAULT_FILTER
 
-            allow_create = '%s_allow_create' % self.options_name
-            self.allow_create = getattr(conf.ldap, allow_create)
-
-            allow_update = '%s_allow_update' % self.options_name
-            self.allow_update = getattr(conf.ldap, allow_update)
-
-            allow_delete = '%s_allow_delete' % self.options_name
-            self.allow_delete = getattr(conf.ldap, allow_delete)
-
             member_attribute = '%s_member_attribute' % self.options_name
             self.member_attribute = getattr(conf.ldap, member_attribute, None)
 
@@ -1207,40 +1178,27 @@ class BaseLdap(object):
             attribute_ignore = '%s_attribute_ignore' % self.options_name
             self.attribute_ignore = getattr(conf.ldap, attribute_ignore)
 
-        self.use_dumb_member = conf.ldap.use_dumb_member
-        self.dumb_member = (conf.ldap.dumb_member or
-                            self.DUMB_MEMBER_DN)
-
-        self.subtree_delete_enabled = conf.ldap.allow_subtree_delete
-
     def _not_found(self, object_id):
         if self.NotFound is None:
             return exception.NotFound(target=object_id)
         else:
             return self.NotFound(**{self.notfound_arg: object_id})
 
-    def _parse_extra_attrs(self, option_list):
+    @staticmethod
+    def _parse_extra_attrs(option_list):
         mapping = {}
         for item in option_list:
             try:
                 ldap_attr, attr_map = item.split(':')
-            except Exception:
-                LOG.warning(_LW(
+            except ValueError:
+                LOG.warning(
                     'Invalid additional attribute mapping: "%s". '
-                    'Format must be <ldap_attribute>:<keystone_attribute>'),
+                    'Format must be <ldap_attribute>:<keystone_attribute>',
                     item)
                 continue
             mapping[ldap_attr] = attr_map
         return mapping
 
-    def _is_dumb_member(self, member_dn):
-        """Check that member is a dumb member.
-
-        :param member_dn: DN of member to be checked.
-        """
-        return (self.use_dumb_member
-                and is_dn_equal(member_dn, self.dumb_member))
-
     def get_connection(self, user=None, password=None, end_user_auth=False):
         use_pool = self.use_pool
         pool_size = self.pool_size
@@ -1258,35 +1216,46 @@ class BaseLdap(object):
 
         conn = KeystoneLDAPHandler(conn=conn)
 
-        conn.connect(self.LDAP_URL,
-                     page_size=self.page_size,
-                     alias_dereferencing=self.alias_dereferencing,
-                     use_tls=self.use_tls,
-                     tls_cacertfile=self.tls_cacertfile,
-                     tls_cacertdir=self.tls_cacertdir,
-                     tls_req_cert=self.tls_req_cert,
-                     chase_referrals=self.chase_referrals,
-                     debug_level=self.debug_level,
-                     use_pool=use_pool,
-                     pool_size=pool_size,
-                     pool_retry_max=self.pool_retry_max,
-                     pool_retry_delay=self.pool_retry_delay,
-                     pool_conn_timeout=self.pool_conn_timeout,
-                     pool_conn_lifetime=pool_conn_lifetime
-                     )
+        # The LDAP server may be down or a connection may not
+        # exist. If that is the case, the bind attempt will
+        # fail with a server down exception.
+        try:
+            conn.connect(self.LDAP_URL,
+                         page_size=self.page_size,
+                         alias_dereferencing=self.alias_dereferencing,
+                         use_tls=self.use_tls,
+                         tls_cacertfile=self.tls_cacertfile,
+                         tls_cacertdir=self.tls_cacertdir,
+                         tls_req_cert=self.tls_req_cert,
+                         chase_referrals=self.chase_referrals,
+                         debug_level=self.debug_level,
+                         conn_timeout=self.conn_timeout,
+                         use_pool=use_pool,
+                         pool_size=pool_size,
+                         pool_retry_max=self.pool_retry_max,
+                         pool_retry_delay=self.pool_retry_delay,
+                         pool_conn_timeout=self.pool_conn_timeout,
+                         pool_conn_lifetime=pool_conn_lifetime)
 
-        if user is None:
-            user = self.LDAP_USER
+            if user is None:
+                user = self.LDAP_USER
 
-        if password is None:
-            password = self.LDAP_PASSWORD
+            if password is None:
+                password = self.LDAP_PASSWORD
 
-        # not all LDAP servers require authentication, so we don't bind
-        # if we don't have any user/pass
-        if user and password:
-            conn.simple_bind_s(user, password)
+            # not all LDAP servers require authentication, so we don't bind
+            # if we don't have any user/pass
+            if user and password:
+                conn.simple_bind_s(user, password)
+            else:
+                conn.simple_bind_s()
 
-        return conn
+            return conn
+        except ldap.INVALID_CREDENTIALS:
+            raise exception.LDAPInvalidCredentialsError()
+        except ldap.SERVER_DOWN:
+            raise exception.LDAPServerConnectionError(
+                url=self.LDAP_URL)
 
     def _id_to_dn_string(self, object_id):
         return u'%s=%s,%s' % (self.id_attr,
@@ -1336,11 +1305,10 @@ class BaseLdap(object):
             # logic here so it does not potentially break existing
             # deployments. We need to fix our read-write LDAP logic so
             # it does not get the ID from DN.
-            message = _LW('ID attribute %(id_attr)s for LDAP object %(dn)s '
-                          'has multiple values and therefore cannot be used '
-                          'as an ID. Will get the ID from DN instead') % (
-                              {'id_attr': self.id_attr,
-                               'dn': res[0]})
+            message = ('ID attribute %(id_attr)s for LDAP object %(dn)s '
+                       'has multiple values and therefore cannot be used '
+                       'as an ID. Will get the ID from DN instead') % (
+                           {'id_attr': self.id_attr, 'dn': res[0]})
             LOG.warning(message)
             id_val = self._dn_to_id(res[0])
         else:
@@ -1369,21 +1337,6 @@ class BaseLdap(object):
 
         return obj
 
-    def check_allow_create(self):
-        if not self.allow_create:
-            action = _('LDAP %s create') % self.options_name
-            raise exception.ForbiddenAction(action=action)
-
-    def check_allow_update(self):
-        if not self.allow_update:
-            action = _('LDAP %s update') % self.options_name
-            raise exception.ForbiddenAction(action=action)
-
-    def check_allow_delete(self):
-        if not self.allow_delete:
-            action = _('LDAP %s delete') % self.options_name
-            raise exception.ForbiddenAction(action=action)
-
     def affirm_unique(self, values):
         if values.get('name') is not None:
             try:
@@ -1428,12 +1381,57 @@ class BaseLdap(object):
                 for attr in extra_attrs:
                     attrs.append((attr, [v]))
 
-        if 'groupOfNames' in object_classes and self.use_dumb_member:
-            attrs.append(('member', [self.dumb_member]))
         with self.get_connection() as conn:
             conn.add_s(self._id_to_dn(values['id']), attrs)
         return values
 
+    # NOTE(prashkre): Filter ldap search results on an attribute to ensure
+    # that attribute has a value set on ldap. This keeps keystone away
+    # from entities that don't have attribute value set on ldap.
+    # for e.g. In ldap configuration, if user_name_attribute = personName
+    # then it will ignore ldap users who don't have 'personName' attribute
+    # value set on user.
+    def _filter_ldap_result_by_attr(self, ldap_result, ldap_attr_name):
+        attr = self.attribute_mapping[ldap_attr_name]
+
+        # To ensure that ldap attribute value is not empty in ldap config.
+        if not attr:
+            attr_name = ('%s_%s_attribute' %
+                         (self.options_name,
+                          self.attribute_options_names[ldap_attr_name]))
+            raise ValueError('"%(attr)s" is not a valid value for'
+                             ' "%(attr_name)s"' % {'attr': attr,
+                                                   'attr_name': attr_name})
+
+        # consider attr = "cn" and
+        # ldap_result = [{'uid': ['fake_id1']}, , 'cN': ["name"]}]
+        # doing lower case on both user_name_attribute and ldap users
+        # attribute
+        result = []
+        # consider attr = "cn" and
+        # ldap_result = [(u'cn=fake1,o=ex_domain', {'uid': ['fake_id1']}),
+        #                (u'cn=fake2,o=ex_domain', {'uid': ['fake_id2'],
+        #                'cn': ['     ']}),
+        #                (u'cn=fake3,o=ex_domain', {'uid': ['fake_id3'],
+        #                'cn': ['']}),
+        #                (u'cn=fake4,o=ex_domain', {'uid': ['fake_id4'],
+        #                'cn': []}),
+        #                (u'cn=fake5,o=ex_domain', {'uid': ['fake_id5'],
+        #                'cn': ["name"]})]
+        for obj in ldap_result:
+            # ignore ldap object(user/group entry) which has no attr set
+            # in it or whose value is empty list.
+            ldap_res_low_keys_dict = {k.lower(): v for k, v in obj[1].items()}
+            result_attr_vals = ldap_res_low_keys_dict.get(attr.lower())
+            # ignore ldap object whose attr value has empty strings or
+            # contains only whitespaces.
+            if result_attr_vals:
+                if result_attr_vals[0] and result_attr_vals[0].strip():
+                    result.append(obj)
+        # except {'uid': ['fake_id5'], 'cn': ["name"]}, all entries
+        # will be ignored in ldap_result
+        return result
+
     def _ldap_get(self, object_id, ldap_filter=None):
         query = (u'(&(%(id_attr)s=%(id)s)'
                  u'%(filter)s'
@@ -1454,8 +1452,16 @@ class BaseLdap(object):
                                     attrs)
             except ldap.NO_SUCH_OBJECT:
                 return None
+
+        # TODO(prashkre): add functional testing for missing name attibute
+        # on ldap entities.
+        # NOTE(prashkre): Filter ldap search result to keep keystone away from
+        # entities that don't have names. We can also do the same by appending
+        # a condition '(!(!(self.attribute_mapping.get('name')=*))' to ldap
+        # search query but the repsonse time of the query is pretty slow when
+        # compared to explicit filtering by 'name' through ldap result.
         try:
-            return res[0]
+            return self._filter_ldap_result_by_attr(res[:1], 'name')[0]
         except IndexError:
             return None
 
@@ -1485,19 +1491,28 @@ class BaseLdap(object):
                           list(self.extra_attr_mapping.keys()))))
         if hints.limit:
             sizelimit = hints.limit['limit']
-            return self._ldap_get_limited(self.tree_dn,
-                                          self.LDAP_SCOPE,
-                                          query,
-                                          attrs,
-                                          sizelimit)
-        with self.get_connection() as conn:
-            try:
-                return conn.search_s(self.tree_dn,
-                                     self.LDAP_SCOPE,
-                                     query,
-                                     attrs)
-            except ldap.NO_SUCH_OBJECT:
-                return []
+            res = self._ldap_get_limited(self.tree_dn,
+                                         self.LDAP_SCOPE,
+                                         query,
+                                         attrs,
+                                         sizelimit)
+        else:
+            with self.get_connection() as conn:
+                try:
+                    res = conn.search_s(self.tree_dn,
+                                        self.LDAP_SCOPE,
+                                        query,
+                                        attrs)
+                except ldap.NO_SUCH_OBJECT:
+                    return []
+        # TODO(prashkre): add functional testing for missing name attribute
+        # on ldap entities.
+        # NOTE(prashkre): Filter ldap search result to keep keystone away from
+        # entities that don't have names. We can also do the same by appending
+        # a condition '(!(!(self.attribute_mapping.get('name')=*))' to ldap
+        # search query but the repsonse time of the query is pretty slow when
+        # compared to explicit filtering by 'name' through ldap result.
+        return self._filter_ldap_result_by_attr(res, 'name')
 
     def _ldap_get_list(self, search_base, scope, query_params=None,
                        attrlist=None):
@@ -1589,50 +1604,6 @@ class BaseLdap(object):
 
         return self.get(object_id)
 
-    def delete(self, object_id):
-        with self.get_connection() as conn:
-            try:
-                conn.delete_s(self._id_to_dn(object_id))
-            except ldap.NO_SUCH_OBJECT:
-                raise self._not_found(object_id)
-
-    def delete_tree(self, object_id):
-        tree_delete_control = ldap.controls.LDAPControl(CONTROL_TREEDELETE,
-                                                        0,
-                                                        None)
-        with self.get_connection() as conn:
-            try:
-                conn.delete_ext_s(self._id_to_dn(object_id),
-                                  serverctrls=[tree_delete_control])
-            except ldap.NO_SUCH_OBJECT:
-                raise self._not_found(object_id)
-            except ldap.NOT_ALLOWED_ON_NONLEAF:
-                # Most LDAP servers do not support the tree_delete_control.
-                # In these servers, the usual idiom is to first perform a
-                # search to get the entries to delete, then delete them
-                # in order of child to parent, since LDAP forbids the
-                # deletion of a parent entry before deleting the children
-                # of that parent.  The simplest way to do that is to delete
-                # the entries in order of the length of the DN, from longest
-                # to shortest DN.
-                dn = self._id_to_dn(object_id)
-                scope = ldap.SCOPE_SUBTREE
-                # With some directory servers, an entry with objectclass
-                # ldapsubentry will not be returned unless it is explicitly
-                # requested, by specifying the objectclass in the search
-                # filter.  We must specify this, with objectclass=*, in an
-                # LDAP filter OR clause, in order to return all entries
-                filt = '(|(objectclass=*)(objectclass=ldapsubentry))'
-                # We only need the DNs of the entries.  Since no attributes
-                # will be returned, we do not have to specify attrsonly=1.
-                entries = conn.search_s(dn, scope, filt, attrlist=DN_ONLY)
-                if entries:
-                    for dn in sorted((e[0] for e in entries),
-                                     key=len, reverse=True):
-                        conn.delete_s(dn)
-                else:
-                    LOG.debug('No entries in LDAP subtree %s', dn)
-
     def add_member(self, member_dn, member_list_dn):
         """Add member to the member list.
 
@@ -1651,59 +1622,11 @@ class BaseLdap(object):
                 raise exception.Conflict(_('Member %(member)s '
                                            'is already a member'
                                            ' of group %(group)s') % {
-                                         'member': member_dn,
-                                         'group': member_list_dn})
+                                               'member': member_dn,
+                                               'group': member_list_dn})
             except ldap.NO_SUCH_OBJECT:
                 raise self._not_found(member_list_dn)
 
-    def remove_member(self, member_dn, member_list_dn):
-        """Remove member from the member list.
-
-        :param member_dn: DN of member to be removed.
-        :param member_list_dn: DN of group from which the
-                               member will be removed.
-
-        :raises self.NotFound: If the group entry didn't exist.
-        :raises ldap.NO_SUCH_ATTRIBUTE: If the user wasn't a member.
-        """
-        with self.get_connection() as conn:
-            try:
-                mod = (ldap.MOD_DELETE, self.member_attribute, member_dn)
-                conn.modify_s(member_list_dn, [mod])
-            except ldap.NO_SUCH_OBJECT:
-                raise self._not_found(member_list_dn)
-
-    def _delete_tree_nodes(self, search_base, scope, query_params=None):
-        query = u'(objectClass=%s)' % self.object_class
-        if query_params:
-            query = (u'(&%s%s)' %
-                     (query, ''.join(['(%s=%s)'
-                                      % (k, ldap.filter.escape_filter_chars(v))
-                                      for k, v in
-                                      query_params.items()])))
-        not_deleted_nodes = []
-        with self.get_connection() as conn:
-            try:
-                nodes = conn.search_s(search_base, scope, query,
-                                      attrlist=DN_ONLY)
-            except ldap.NO_SUCH_OBJECT:
-                LOG.debug('Could not find entry with dn=%s', search_base)
-                raise self._not_found(self._dn_to_id(search_base))
-            else:
-                for node_dn, _t in nodes:
-                    try:
-                        conn.delete_s(node_dn)
-                    except ldap.NO_SUCH_OBJECT:
-                        not_deleted_nodes.append(node_dn)
-
-        if not_deleted_nodes:
-            LOG.warning(_LW("When deleting entries for %(search_base)s, "
-                            "could not delete nonexistent entries "
-                            "%(entries)s%(dots)s"),
-                        {'search_base': search_base,
-                         'entries': not_deleted_nodes[:3],
-                         'dots': '...' if len(not_deleted_nodes) > 3 else ''})
-
     def filter_query(self, hints, query=None):
         """Apply filtering to a query.
 
@@ -1717,11 +1640,10 @@ class BaseLdap(object):
         :returns query: LDAP query, updated with any filters satisfied
 
         """
-        def build_filter(filter_, hints):
+        def build_filter(filter_):
             """Build a filter for the query.
 
             :param filter_: the dict that describes this filter
-            :param hints: contains the list of filters yet to be satisfied.
 
             :returns query: LDAP query term to be added
 
@@ -1785,7 +1707,7 @@ class BaseLdap(object):
         for filter_ in hints.filters:
             if filter_['name'] not in self.attribute_mapping:
                 continue
-            new_filter = build_filter(filter_, hints)
+            new_filter = build_filter(filter_)
             if new_filter is not None:
                 filter_list.append(new_filter)
                 satisfied_filters.append(filter_)
@@ -1879,8 +1801,6 @@ class EnabledEmuMixIn(BaseLdap):
                                  (self.member_attribute,
                                   [self._id_to_dn(object_id)]),
                                  self.enabled_emulation_naming_attr]
-                    if self.use_dumb_member:
-                        attr_list[1][1].append(self.dumb_member)
                     conn.add_s(self.enabled_emulation_dn, attr_list)
 
     def _remove_enabled(self, object_id):
@@ -1944,8 +1864,3 @@ class EnabledEmuMixIn(BaseLdap):
         else:
             return super(EnabledEmuMixIn, self).update(
                 object_id, values, old_obj)
-
-    def delete(self, object_id):
-        if self.enabled_emulation:
-            self._remove_enabled(object_id)
-        super(EnabledEmuMixIn, self).delete(object_id)
